#include <algorithm>
#include <iostream>
#include <cassert>
#include <vector>
#include <set>
using namespace std;

#define INPUTS 3

class Gate {
    bool is_monotonic() const;
    bool is_affine() const;
    bool is_self_dual() const;
    bool is_truth_preserving() const;
    bool is_falsity_preserving() const;
    public:
    vector<bool> f; // f[x] is the output of the gate if the inputs are the bits of x
    Gate(int x);
    bool evaluate(const vector<bool> &inputs) const;
    bool evaluate(int v) const;
    bool is_universal() const;
};

Gate::Gate(int x=0) { f.resize(1<<INPUTS); for (int i=0; i<(1<<INPUTS); ++i) f[i] = (x>>i)&1; }

bool Gate::evaluate(const vector<bool> &inputs) const {
    int key = 0;
    for (int i=0; i<INPUTS; ++i) if (inputs[i]) key |= 1<<i;
    return f[key];
}

bool Gate::evaluate(int v) const { return f[v]; }

bool Gate::is_monotonic() const {
    for (int i=0; i<(1<<INPUTS); ++i) {
        bool v = evaluate(i);
        if (!v) continue;
        for (int j=0; j<INPUTS; ++j) if (!(i & 1<<j)) {
            bool w = evaluate(i | 1<<j);
            if (!w) return false;
        }
    }
    return true;
}

bool Gate::is_affine() const {
    for (int j=0; j<INPUTS; ++j) {
        bool seen_same=false, seen_different=false;
        for (int i=0; i<(1<<INPUTS); ++i) {
            bool v = evaluate(i);
            bool w = evaluate(i ^ 1<<j);
            if (v==w) seen_same=true; else seen_different=true;
        }
        if (seen_same && seen_different) return false;
    }
    return true;
}

bool Gate::is_self_dual() const {
    for (int i=0; i<(1<<INPUTS); ++i) {
        int noti = (~i) & ((1<<INPUTS)-1);
        bool v = evaluate(i);
        bool w = evaluate(noti);
        if (v==w) return false;
    }
    return true;
}

bool Gate::is_truth_preserving() const {
    return evaluate( (1<<INPUTS)-1 ) == true;
}

bool Gate::is_falsity_preserving() const {
    return evaluate(0) == false;
}

bool Gate::is_universal() const {
    if (is_monotonic()) return false;
    if (is_affine()) return false;
    if (is_self_dual()) return false;
    if (is_truth_preserving()) return false;
    if (is_falsity_preserving()) return false;
    return true;
}

int main() {
    for (int vals=0; vals<(1<<(1<<INPUTS)); ++vals) 
        if (Gate(vals).is_universal())
            for (int i=0; i<(1<<INPUTS); ++i) 
                cout << ((vals >> i)&1) << (i+1==(1<<INPUTS)?'\n':' ');
}
